<?php
/**
 * initialization class for tx_date2cal
 */

if(!defined('TYPO3_MODE'))
	die('Access denied.');

if(!class_exists('tx_date2cal_extTables')) {
	/** initialization class for tx_date2cal */
	class tx_date2cal_extTables
	{

		/** @var array holds configuration */
		var $extConf;

		/** @var string content of cache file */
		var $cacheFile;

		/** @var string cache file name */
		var $cacheFileName;

		/** @var array used tca tables (needed as cache information) */
		var $tcaTables;

		/**
		 * Constructor
		 * :: initialize tx_date2cal
		 * :: loads the extension configuration
		 *
		 * @return void
		 */
		function tx_date2cal_extTables()
		{
			// init variables and configuration
			$this->extConf = unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['date2cal']);
			$this->extConf['extCache'] = $GLOBALS['TYPO3_CONF_VARS']['EXT']['extCache'];

			if(!empty($GLOBALS['TYPO3_LOADED_EXT']['_CACHEFILE']))
				$this->cacheFileName = PATH_typo3conf . $GLOBALS['TYPO3_LOADED_EXT']['_CACHEFILE'] . '_ext_tables.php';

			// convert endtime/startime fields to datetime
			if($this->extConf['datetime'])
				$this->tx_date2cal_toDatetime();
			$this->tx_date2cal_setWizard();

			// write cache file (only if the cache file was loaded already)
			if((in_array($this->cacheFileName, get_required_files()) ||
				in_array($this->cacheFileName, get_included_files())) &&
				$this->extConf['extCache'] && $this->extConf['doCache'])
			{
				$this->tx_date2cal_writeCacheFile();
			}
		}

		/**
		 * writes the cache file
		 *
		 * @return void
		 */
		function tx_date2cal_writeCacheFile()
		{
			foreach($this->tcaTables as $tcaTable=>$bool)
				$this->cacheFile = 't3lib_div::loadTCA(\'' . $tcaTable . '\');' . "\n" . $this->cacheFile;

			// save cache file
			$extTablesCnt = t3lib_div::getURL($this->cacheFileName);
			$initLine = '$GLOBALS[\'tx_date2cal_init\'] = true;' . "\n";
			$this->cacheFile = '<?php' . "\n" . $initLine . '?>' . $extTablesCnt . '<?php' . "\n" . $this->cacheFile . '?>';
			t3lib_div::writeFile(PATH_typo3conf . $GLOBALS['TYPO3_LOADED_EXT']['_CACHEFILE'] . '_ext_tables.php', $this->cacheFile);
		}

		/**
		 * sets date2cal wizard to each date/datetime field
		 *
		 * @return array tca tables
		 */
		function tx_date2cal_setWizard()
		{
			foreach($GLOBALS['TCA'] as $myTable => $tConf) {
				t3lib_div::loadTCA($myTable);
				$changed = false;
				foreach($GLOBALS['TCA'][$myTable]['columns'] as $field => $fConf) {
					if(($fConf['config']['type'] == 'input') &&
					   (t3lib_div::inList($fConf['config']['eval'], 'date') ||
					    t3lib_div::inList($fConf['config']['eval'], 'datetime'))) {

						// add calendar (needed if caching is disabled)
						$GLOBALS['TCA'][$myTable]['columns'][$field]['config']['wizards']['calendar']['type'] = 'userFunc';
						$GLOBALS['TCA'][$myTable]['columns'][$field]['config']['wizards']['calendar']['userFunc'] =
							'EXT:date2cal/class.tx_date2cal_wizard.php:tx_date2cal_wizard->renderWizard';
						$GLOBALS['TCA'][$myTable]['columns'][$field]['config']['wizards']['calendar']['evalValue'] =
							$fConf['config']['eval'];

						$this->cacheFile .= '$GLOBALS[\'TCA\'][\'' . $myTable . '\'][\'columns\'][\'' . $field .
							'\'][\'config\'][\'wizards\'][\'calendar\'][\'type\'] = \'userFunc\';' . "\n";
						$this->cacheFile .= '$GLOBALS[\'TCA\'][\'' . $myTable . '\'][\'columns\'][\'' . $field .
							'\'][\'config\'][\'wizards\'][\'calendar\'][\'userFunc\'] = ' .
							'\'EXT:date2cal/class.tx_date2cal_wizard.php:tx_date2cal_wizard->renderWizard\';' . "\n";
						$this->cacheFile .= '$GLOBALS[\'TCA\'][\'' . $myTable . '\'][\'columns\'][\'' . $field .
							'\'][\'config\'][\'wizards\'][\'calendar\'][\'evalValue\'] = ' .
							'\'' . $fConf['config']['eval'] . '\';' . "\n";
						$changed = true;
					}
				}
				if($changed)
					$this->tcaTables[$myTable] = $changed;
			}
		}

		/**
		 * forces starttime/endtime fields to be datetime fields (just overrides default values)
		 * Note: this is only done for tt_content and pages table
		 *
		 * @return void
		 */
		function tx_date2cal_toDatetime()
		{
		    t3lib_div::loadTCA('tt_content');
			$GLOBALS['TCA']['tt_content']['columns']['starttime']['config']['eval'] = 'datetime';
			$GLOBALS['TCA']['tt_content']['columns']['starttime']['config']['size'] = 12;
			$GLOBALS['TCA']['tt_content']['columns']['endtime']['config']['eval'] = 'datetime';
			$GLOBALS['TCA']['tt_content']['columns']['endtime']['config']['size'] = 12;

		    t3lib_div::loadTCA('pages');
			$GLOBALS['TCA']['pages']['columns']['starttime']['config']['eval'] = 'datetime';
			$GLOBALS['TCA']['pages']['columns']['starttime']['config']['size'] = 12;
			$GLOBALS['TCA']['pages']['columns']['endtime']['config']['eval'] = 'datetime';
			$GLOBALS['TCA']['pages']['columns']['endtime']['config']['size'] = 12;

			$this->cacheFile = '$GLOBALS[\'TCA\'][\'tt_content\'][\'columns\'][\'starttime\'][\'config\'][\'eval\'] = \'datetime\';' . "\n";
			$this->cacheFile .= '$GLOBALS[\'TCA\'][\'tt_content\'][\'columns\'][\'starttime\'][\'config\'][\'size\'] = 12;' . "\n";
			$this->cacheFile .= '$GLOBALS[\'TCA\'][\'tt_content\'][\'columns\'][\'endtime\'][\'config\'][\'eval\'] = \'datetime\';' . "\n";
			$this->cacheFile .= '$GLOBALS[\'TCA\'][\'tt_content\'][\'columns\'][\'endtime\'][\'config\'][\'size\'] = 12;' . "\n";

			$this->cacheFile .= '$GLOBALS[\'TCA\'][\'pages\'][\'columns\'][\'starttime\'][\'config\'][\'eval\'] = \'datetime\';' . "\n";
			$this->cacheFile .= '$GLOBALS[\'TCA\'][\'pages\'][\'columns\'][\'starttime\'][\'config\'][\'size\'] = 12;' . "\n";
			$this->cacheFile .= '$GLOBALS[\'TCA\'][\'pages\'][\'columns\'][\'endtime\'][\'config\'][\'eval\'] = \'datetime\';' . "\n";
			$this->cacheFile .= '$GLOBALS[\'TCA\'][\'pages\'][\'columns\'][\'endtime\'][\'config\'][\'size\'] = 12;' . "\n";

			$this->tcaTables['tt_content'] = true;
			$this->tcaTables['pages'] = true;
		}
	}

	// exec class
	if(!$GLOBALS['tx_date2cal_init']) {
        $date2cal = new tx_date2cal_extTables();
        unset($date2cal);
    }
}
?>
